package com.aks_labs.tulsi.compose.grids

import android.net.Uri
import androidx.activity.compose.BackHandler
import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.fadeIn
import androidx.compose.animation.fadeOut
import androidx.compose.animation.slideInVertically
import androidx.compose.animation.slideOutVertically
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.WindowInsets
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.navigationBars
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.windowInsetsPadding
import androidx.compose.material3.BottomSheetScaffold
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SheetValue
import androidx.compose.material3.rememberBottomSheetScaffoldState
import androidx.compose.material3.rememberStandardBottomSheetState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.derivedStateOf
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.runtime.snapshots.SnapshotStateList
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.RectangleShape
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalLayoutDirection
import androidx.compose.ui.unit.dp
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import androidx.lifecycle.viewmodel.compose.viewModel
import com.aks_labs.tulsi.LocalNavController
import com.aks_labs.tulsi.MainActivity.Companion.mainViewModel
import com.aks_labs.tulsi.compose.ViewProperties
import com.aks_labs.tulsi.compose.app_bars.TrashedPhotoGridViewBottomBar
import com.aks_labs.tulsi.compose.app_bars.TrashedPhotoGridViewTopBar
import com.aks_labs.tulsi.datastore.AlbumInfo
import com.aks_labs.tulsi.datastore.BottomBarTab
import com.aks_labs.tulsi.datastore.TrashBin
import com.aks_labs.tulsi.helpers.permanentlyDeletePhotoList
import com.aks_labs.tulsi.mediastore.MediaStoreData
import com.aks_labs.tulsi.mediastore.MediaType
import com.aks_labs.tulsi.models.trash_bin.TrashViewModel
import com.aks_labs.tulsi.models.trash_bin.TrashViewModelFactory
import kotlinx.coroutines.Dispatchers
import kotlin.time.Duration.Companion.days

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun TrashedPhotoGridView(
    selectedItemsList: SnapshotStateList<MediaStoreData>,
    currentView: MutableState<BottomBarTab>
) {
    val context = LocalContext.current
    val trashViewModel: TrashViewModel = viewModel(
        factory = TrashViewModelFactory(context = context)
    )

    val mediaStoreData =
        trashViewModel.mediaFlow.collectAsStateWithLifecycle(context = Dispatchers.IO)

    val groupedMedia = remember { mutableStateOf(mediaStoreData.value) }

    LaunchedEffect(mediaStoreData.value) {
        groupedMedia.value = mediaStoreData.value
    }

    var triedDeletingAlready by rememberSaveable { mutableStateOf(false) }
    val autoDeleteInterval by mainViewModel.settings.TrashBin.getAutoDeleteInterval()
        .collectAsStateWithLifecycle(initialValue = 0)

    val runAutoDeleteAction = remember { mutableStateOf(false) }
    var mediaToBeAutoDeleted by remember { mutableStateOf(emptyList<Uri>()) }

    LaunchedEffect(runAutoDeleteAction.value) {
        permanentlyDeletePhotoList(context, mediaToBeAutoDeleted)

        triedDeletingAlready = true
        runAutoDeleteAction.value = false
    }

    LaunchedEffect(groupedMedia.value, autoDeleteInterval) {
        if (groupedMedia.value.isEmpty() || triedDeletingAlready || autoDeleteInterval == 0) return@LaunchedEffect

        val currentDate = System.currentTimeMillis()

        mediaToBeAutoDeleted = groupedMedia.value
            .filter { it.type != MediaType.Section }
            .filter { media ->
                val dateDeletedMillis =
                    currentDate - (media.dateModified * 1000) // dateModified is in seconds
                val dateDeletedDays = (dateDeletedMillis / (1000 * 60 * 60 * 24)).days

                dateDeletedDays > autoDeleteInterval.days
            }
            .map {
                it.uri
            }

        runAutoDeleteAction.value = true
    }

    val showBottomSheet by remember {
        derivedStateOf {
            selectedItemsList.isNotEmpty()
        }
    }

    BackHandler(
        enabled = showBottomSheet
    ) {
        selectedItemsList.clear()
    }

    val navController = LocalNavController.current
    BackHandler(
        enabled = !showBottomSheet
    ) {
        navController.popBackStack()
    }

    Scaffold(
        topBar = {
            TrashedPhotoGridViewTopBar(
                selectedItemsList = selectedItemsList,
                groupedMedia = groupedMedia.value,
                currentView = currentView
            ) {
                navController.popBackStack()
            }
        },
        bottomBar = {
            AnimatedVisibility(
                visible = showBottomSheet,
                enter = slideInVertically { it } + fadeIn(),
                exit = slideOutVertically { it } + fadeOut()
            ) {
                TrashedPhotoGridViewBottomBar(selectedItemsList = selectedItemsList)
            }
        },
        containerColor = Color.Transparent,
        contentColor = MaterialTheme.colorScheme.onBackground,
        modifier = Modifier.fillMaxSize(1f)
    ) { padding ->
        Column(
            modifier = Modifier
                .padding(
                    start = padding.calculateLeftPadding(LocalLayoutDirection.current),
                    top = padding.calculateTopPadding(),
                    end = padding.calculateRightPadding(LocalLayoutDirection.current),
                    bottom = 0.dp // Remove bottom padding to allow content to be visible behind the bottom bar
                )
                .fillMaxSize(1f)
                .windowInsetsPadding(
                    WindowInsets.navigationBars
                ),
            verticalArrangement = Arrangement.Center,
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            PhotoGrid(
                groupedMedia = groupedMedia,
                albumInfo = AlbumInfo.createPathOnlyAlbum(emptyList()),
                selectedItemsList = selectedItemsList,
                viewProperties = ViewProperties.Trash,
                shouldPadUp = true
            )
        }
    }
}


